//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.AdvancedViewers
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			_calendar.InteractiveItemType = typeof(TreeAppointment);
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void calendar_ItemClick(object sender, MindFusion.Scheduling.WinForms.ItemMouseEventArgs e)
		{
			TreeAppointment app = e.Item as TreeAppointment;
			TreeViewEx view = app.View;

			Win32.LPTVHITTESTINFO info = new Win32.LPTVHITTESTINFO();
			info.pt.x = e.X;
			info.pt.y = e.Y;

			Win32.SendMessage(view.Handle, Win32.TVM_HITTEST, 0, ref info);

			if (info.hItem != IntPtr.Zero)
			{
				if (e.Button == MouseButtons.Left)
				{
					if ((info.flags & Win32.TVHT_ONITEMBUTTON) > 0)
					{
						// +/- icon has been hit-tested
						_calendar.ResetDrag();

						TreeNode node = view.NodeFromHandle(info.hItem);
						if (node.IsExpanded)
							node.Collapse();
						else
							node.Expand();

						_calendar.Invalidate();
					}

					else if ((info.flags & Win32.TVHT_ONITEMLABEL) > 0)
					{
						// Select the clicked item
						_calendar.ResetDrag();

						TreeNode node = view.NodeFromHandle(info.hItem);
						view.SelectedNode = node;

						_calendar.Invalidate();

						// Start dragging the selected node
						DoDragDrop(new TreeDragData(app, node), DragDropEffects.Move);
					}
				}

				else if (e.Button == MouseButtons.Right)
				{
					// Select the clicked item
					TreeNode node = view.NodeFromHandle(info.hItem);
					view.SelectedNode = node;

					_calendar.Invalidate();

					// Remember the clicked node
					_contextNode = node;
					_contextItem = app;

					// Display context menu for the clicked item
					_contextMenu.Show(_calendar, _calendar.PointToClient(Cursor.Position));
				}
			}
		}

		private void _calendar_DragOver(object sender, System.Windows.Forms.DragEventArgs e)
		{
			Point point = _calendar.PointToClient(new Point(e.X, e.Y));

			Item item = _calendar.GetItemAt(point);
			if (item == null)
			{
				e.Effect = DragDropEffects.None;
				return;
			}

			if (!e.Data.GetDataPresent(typeof(TreeDragData)))
			{
				e.Effect = DragDropEffects.None;
				return;
			}

			TreeDragData data = e.Data.GetData(typeof(TreeDragData)) as TreeDragData;
			if (data.Item == item)
			{
				e.Effect = DragDropEffects.None;
				return;
			}

			e.Effect = DragDropEffects.Move;
		}

		private void _calendar_DragDrop(object sender, System.Windows.Forms.DragEventArgs e)
		{
			Point point = _calendar.PointToClient(new Point(e.X, e.Y));

			Item item = _calendar.GetItemAt(point);
			if (item == null)
				return;

			if (!e.Data.GetDataPresent(typeof(TreeDragData)))
				return;

			TreeDragData data = e.Data.GetData(typeof(TreeDragData)) as TreeDragData;
			if (data.Item == item)
				return;

			TreeAppointment targetItem = item as TreeAppointment;
			TreeAppointment sourceItem = data.Item as TreeAppointment;
			TreeNode draggedNode = data.Node;

			// Remove the dragged node from its parent
			if (draggedNode.Parent != null)
				draggedNode.Parent.Nodes.Remove(draggedNode);
			else
				sourceItem.View.Nodes.Remove(draggedNode);

			// Add the dragged node to the target item
			Rectangle targetItemBounds = _calendar.GetItemBounds(item);
			Win32.LPTVHITTESTINFO info = new Win32.LPTVHITTESTINFO();
			info.pt.x = point.X - targetItemBounds.X;
			info.pt.y = point.Y - targetItemBounds.Y;

			Win32.SendMessage(targetItem.View.Handle, Win32.TVM_HITTEST, 0, ref info);

			if (info.hItem != IntPtr.Zero)
			{
				// The drop has occurred on a node in the target view.
				// Add the dragged node as a child to the pointed node
				TreeNode targetNode = targetItem.View.NodeFromHandle(info.hItem);
				targetNode.Nodes.Add(draggedNode);
			}
			else
			{
				// The drop has occurred on an area in the target view,
				// which is not occuppied by a node. Add the dragged node
				// as a root in the target view
				targetItem.View.Nodes.Add(draggedNode);
			}

			// Use this trick to force the view to repaint itself.
			// Otherwise, if it has no items before this item was
			// added, it will not repaint for some reason
			targetItem.View.Width -= 1;

			// Invalidate
			_calendar.Invalidate();
		}

		private void _menuInsertNode_Click(object sender, System.EventArgs e)
		{
			InsertNodeForm form = new InsertNodeForm();

			if (form.ShowDialog(this) == DialogResult.OK)
			{
				_contextNode.Nodes.Add(form.NodeText);
				_calendar.Invalidate();
			}
		}

		private void _menuDeleteNode_Click(object sender, System.EventArgs e)
		{
			if (MessageBox.Show(this, "Are you sure you want to delete the selected " +
				"node?", "Confirm Delete", MessageBoxButtons.OKCancel) == DialogResult.OK)
			{
				if (_contextNode.Parent != null)
					_contextNode.Parent.Nodes.Remove(_contextNode);
				else
					_contextItem.View.Nodes.Remove(_contextNode);

				_calendar.Invalidate();
			}
		}


		private string _appPath;
		private TreeNode _contextNode;
		private TreeAppointment _contextItem;
	}
}
